<?php
/**
 * The model file of push module of XXB.
 *
 * @copyright   Copyright 2009-2020 青岛易软天创网络科技有限公司(QingDao Nature Easy Soft Network Technology Co,LTD, www.cnezsoft.com)
 * @license     ZOSL (https://zpl.pub/page/zoslv1.html)
 * @author      xuanxuan
 * @package     push
 * @version     $Id:
 * @link        https://xuanim.com
 */
?>
<?php
require_once(dirname(__FILE__) . '/../../lib/xinge/XingeApp.php');
class pushModel extends model
{
    /**
     * Set push config, android or ios.
     *
     * @param  string  $type android|ios
     * @access public
     * @return array
     */
    public function setPushConfig($type)
    {
        $errors = array();
        $post   = fixer::input('post')
            ->setIF($this->post->accessId != '', 'accessId', trim($this->post->accessId))
            ->setIF($this->post->accessKey != '', 'accessKey', trim($this->post->accessKey))
            ->get();

        if(empty($post->accessId))  $errors['accessId'][]  = $this->lang->push->accessIdEmpty;
        if(empty($post->accessKey)) $errors['accessKey'][] = $this->lang->push->accessKeyEmpty;
        if(!empty($errors)) return array('result' => 'fail', 'message' => $errors);

        $config = json_encode($post);
        if($type == 'android')
        {
            $result = $this->loadModel('setting')->setItem('system.sys.push.android.xinge', $config);
        }
        else if($type == 'ios')
        {
            $result = $this->loadModel('setting')->setItem('system.sys.push.ios.xuan_im', $config);
        }

        if(!$result) return array('result' => 'fail', 'message' => dao::getError());
        return array('result' => 'success', 'message' => $this->lang->saveSuccess, 'locate' => inlink('admin', "type=$type"));
    }

    /**
     * Set push setting.
     *
     * @param  array $setting
     * @access public
     * @return array
     */
    public function setPushSetting($setting = '')
    {
        $this->loadModel('setting');
        if($setting == 'init')
        {
            $enable 	  = 'close';
            $privacyLevel = 1;
        }
        else
        {
            $enable       = isset($setting['enable'])       ? $setting['enable'] : 'close';
            $privacyLevel = isset($setting['privacyLevel']) ? $setting['privacyLevel'] : 1;
        }

        $this->setting->setItem('system.sys.push.common.enable', $enable);
        $this->setting->setItem('system.sys.push.common.privacyLevel', $privacyLevel);
        if(dao::isError()) return array('result' => 'fail', 'message' => dao::getError());
        return array('result' => 'success', 'message' => $this->lang->saveSuccess);
    }

    /**
     * get push config.
     *
     * @access public
     * @return array
     */
    public function getPushConfig()
    {
	    $this->loadModel('setting');
        $push         = new stdClass();

        // return empty object if disabled or not set.
        $enabled      = $this->setting->getItem("owner=system&app=sys&module=push&section=common&key=enable");
        if(empty($enabled) || $enabled == 'close') return $push;

        $ios          = $this->setting->getItem("owner=system&app=sys&module=push&section=ios&key=xuan_im");
        $android      = $this->setting->getItem("owner=system&app=sys&module=push&section=android&key=xinge");
        $privacyLevel = $this->setting->getItem("owner=system&app=sys&module=push&section=common&key=privacyLevel");

        if(!empty($android)) $push->xinge = json_decode($android);
        if(!empty($android) || !empty($ios)) $push->privacyLevel = empty($privacyLevel) ? 0 : (int) $privacyLevel;
        return $push;
    }

    /**
     * Get push List from TABLE_IM_QUEUE.
     *
     * @access public
     * @return array
     */
    public function getPushList()
    {
        $pushList = $this->dao->select('*')->from(TABLE_IM_QUEUE)
            ->where('type')->eq('push')
            ->andWhere('status')->eq('wait')
            ->fetchAll('id');

        if(empty($pushList)) return;
        $this->changeQueueStatus(array_keys($pushList), 'doing');
        return $pushList;
    }

    /**
     * Change queue status.
     *
     * @params array $queueList
     * @params string $status
     * @access public
     * @return array
     */
    public function changeQueueStatus($queueList, $status)
    {
        $processDate = $status == 'done' ? helper::now() : '';
        return $this->dao->update(TABLE_IM_QUEUE)
            ->set('status')->eq($status)
            ->set('processDate')->eq($processDate)
            ->where('id')->in($queueList)
            ->exec();
    }

    /**
     * Save push message to queue.
     *
     * @param  object $message
     * @param  array $receivers
     * @param  object $chat
     * @access public
     * @return array
     */
    public function savePushMessageToQueue($message, $receivers, $chat)
    {
	    $userDeviceInfoList = $this->loadModel('user')->getDeviceInfo($receivers);
        foreach($receivers as $key => $userID)
        {
            $deviceType  = $userDeviceInfoList[$userID]->deviceType;
            $deviceToken = $userDeviceInfoList[$userID]->deviceToken;

            if(empty($deviceType) || empty($deviceToken)) unset($receivers[$key]);
        }
        if(empty($receivers)) return;

        $content = new stdClass();
        $content->sender    = $message->user;
        $content->receivers = implode(',', $receivers);
        $content->cgid      = $message->cgid;

        $queueData = new stdclass();
        $queueData->type    = 'push';
        $queueData->content = json_encode($content);
        $queueData->addDate = helper::now();
        $queueData->result  = '';
        $queueData->status  = 'wait';

        $this->dao->insert(TABLE_IM_QUEUE)->data($queueData)->exec();
    }

    /**
     * Push message
     *
     * @param  int $senderID
     * @param  array $receivers
     * @param  string $cgid
     * @access public
     * @return array
     */
    public function pushMessage($senderID, $receivers, $cgid)
    {
        $deviceList         = array('android' => array(), 'ios' => array());
	    $userDeviceInfoList = $this->loadModel('user')->getDeviceInfo($receivers);
        foreach($receivers as $userID)
        {
            $deviceType  = $userDeviceInfoList[$userID]->deviceType;
            $deviceToken = $userDeviceInfoList[$userID]->deviceToken;

            if(empty($deviceType) || empty($deviceToken)) continue;
            $deviceList[$deviceType][] = $deviceToken;
        }

        $iosResult     = empty($deviceList['ios'])     ? 'iOS device token is empty'     : $this->pushIosMultipleNotification($deviceList['ios'], $senderID, $cgid);
        $androidResult = empty($deviceList['android']) ? 'Android device token is empty' : $this->pushAndroidMultipleNotification($deviceList['android'], $senderID, $cgid);

        return array($androidResult, $iosResult);
    }

    /**
     * push Multiple ios notification.
     *
     * @param  array $iosDeviceList
     * @param  int $senderID
     * @param  string $cgid
     * @access public
     * @return object
     */
    public function pushIosMultipleNotification($iosDeviceList, $senderID, $cgid)
    {
	    $this->loadModel('setting');
        $pushConfig   = $this->setting->getItem("owner=system&app=sys&module=push&section=ios&key=xuan_im");
        $privacyLevel = $this->setting->getItem("owner=system&app=sys&module=push&section=common&key=privacyLevel");
        if(empty($pushConfig)) return 'AccessId or AccessKey is empty';
        $pushConfig  = json_decode($pushConfig);

        /* push notification */
        $pushNotification = $this->getPushNotification($privacyLevel, $senderID, $cgid);

        /* ios push
         *
         * $userID user id in xuan.im
         * $device device token.
         * $token md5($userID + $accessKey + $device + $title)
         *
         * /api-push-userID-deviceToken-title-token
         * */
        $userID    = $pushConfig->accessId;
        $accessKey = $pushConfig->accessKey;
        $title     = $pushNotification;
        $result    = array();
        foreach($iosDeviceList as $iosDevice)
        {
            $token   = md5($accessKey . $userID . $title . $iosDevice);
            $pushUrl = sprintf($this->config->push->pushIosUrl, urlencode($userID), urlencode($iosDevice), urlencode($title), urlencode($token));
            $result[$iosDevice] = $this->loadModel('common')->http($pushUrl);
        }

        $this->logPush('Push to iOS devices' . json_encode($iosDeviceList) . ', result: ' . json_encode($result));
        return $result;
    }

    /**
     * push android multiple notification.
     *
     * @param  array $androidDeviceList
     * @param  int $senderID
     * @param  string $cgid
     * @access public
     * @return object
     */
    public function pushAndroidMultipleNotification($androidDeviceList, $senderID, $cgid)
    {
        $pushConfig   = $this->loadModel('setting')->getItem("owner=system&app=sys&module=push&section=android&key=xinge");
        $privacyLevel = $this->setting->getItem("owner=system&app=sys&module=push&section=common&key=privacyLevel");
        if(empty($pushConfig)) return 'AccessId or AccessKey is empty';
        $pushConfig  = json_decode($pushConfig);

        /* push notification */
        $pushNotification = $this->getPushNotification($privacyLevel, $senderID, $cgid);

        /* android push */
        $push = new XingeApp($pushConfig->accessId, $pushConfig->accessKey);
        $msg  = new XGMessage();

        if($privacyLevel != 1)
        {
            $action = new ClickAction();
            $action->setActionType(clickAction::TYPE_INTENT);
            $action->setIntent(sprintf($this->config->push->intentUrl, urlencode($cgid)));
            $msg->setAction($action);
        }

        $msg->setExpireTime(86400);
        $msg->setTitle($pushNotification);
        $msg->setContent(' ');
        $msg->setType(XGMessage::TYPE_NOTIFICATION);
        $msg->setMultiPkg(1);

        $ret = $push->CreateMultipush($msg, XingeApp::IOSENV_DEV);
        if (!($ret['ret_code'] === 0))
        {
            $this->logPush('Push to android devices' . json_encode($androidDeviceList) . ', result: ' . json_encode($result));
            return $ret;
        }
        else
        {
            $result = array();
            array_push($result, $push->PushDeviceListMultiple($ret['result']['push_id'], $androidDeviceList));
            $this->logPush('Push to android devices' . json_encode($androidDeviceList) . ', result: ' . json_encode($result));
            return $result;
        }
    }

    /**
     * Get push notification.
     *
     * @param  int $privacyLevel
     * @param  int $senderID
     * @param  string $cgid
     * @access public
     * @return string
     */
    public function getPushNotification($privacyLevel, $senderID, $cgid)
    {
        switch($privacyLevel)
        {
            case 0:
                // Show sender.
                $pushNotification = $this->getNotificationTitle($senderID, $cgid);
                break;
            default:
                // Do not show sender.
                $pushNotification = $this->lang->push->youHaveNewMessage;
        }

        return $pushNotification;
    }


    /**
     * Get push notification title.
     *
     * @param  int $sender
     * @param  string $cgid
     * @access public
     * @return string
     */
    public function getNotificationTitle($senderID, $cgid)
    {
        $user = $this->loadModel('im')->userGetByID($senderID);
        $chat = $this->im->chatGetByGID($cgid);

        if($chat->type == 'one2one')
        {
            $title = sprintf($this->lang->push->sentYouNewMessage, $user->realname);
        }
        else if($chat->type == 'group' || $chat->type == 'system')
        {
            $title = sprintf($this->lang->push->sentGroupNewMessage, $user->realname, $chat->name);
        }
        else
        {
            return '';
        }
        return $title;
    }

    /**
     * Log push.
     *
     * @param  string    $log
     * @access public
     * @return void
     */
    public function logPush($log)
    {
        if(empty($this->config->debug)) return false;
        $log = "\n" . date('H:i:s') . " $log";
        if(!is_writable($this->app->getLogRoot())) return false;

        $file = $this->app->getLogRoot() . 'push.' . date('Ymd') . '.log.php';
        if(!is_file($file)) $log = "<?php\n die();\n?>\n" . $log;

        $fp = fopen($file, "a");
        fwrite($fp, $log);
        fclose($fp);
    }
}
